@@ -65,30 +65,49 @@ setMethod("columns", "EnsDb",
 ####------------------------------------------------------------
 setMethod("keytypes", "EnsDb",
           function(x){
-              return(.filterKeytypes())
+              return(.filterKeytypes(withProteins = hasProteinData(x)))
           }
 )
 ## This just returns some (eventually) usefull names for keys
 .simpleKeytypes <- function(x){
     return(c("GENEID","TXID","TXNAME","EXONID","EXONNAME","CDSID","CDSNAME"))
 }
-.filterKeytypes <- function(x){
-    return(names(.keytype2FilterMapping()))
+.filterKeytypes <- function(withProteins = FALSE){
+    return(names(.keytype2FilterMapping(withProteins = withProteins)))
 }
 ## returns a vector mapping keytypes (names of vector) to filter names (elements).
-.keytype2FilterMapping <- function(){
-    filters <- c("EntrezidFilter", "GeneidFilter", "GenebiotypeFilter", "GenenameFilter",
-                 "TxidFilter", "TxbiotypeFilter", "ExonidFilter", "SeqnameFilter",
-                 "SeqstrandFilter", "TxidFilter", "SymbolFilter")
-    names(filters) <- c("ENTREZID", "GENEID", "GENEBIOTYPE", "GENENAME", "TXID",
-                        "TXBIOTYPE", "EXONID", "SEQNAME", "SEQSTRAND", "TXNAME",
-                        "SYMBOL")
+.keytype2FilterMapping <- function(withProteins = FALSE){
+    filters <- c(ENTREZID = "EntrezFilter",
+                 GENEID = "GeneIdFilter",
+                 GENEBIOTYPE = "GeneBiotypeFilter",
+                 GENENAME = "GenenameFilter",
+                 TXID = "TxIdFilter",
+                 TXBIOTYPE = "TxBiotypeFilter",
+                 EXONID = "ExonIdFilter",
+                 SEQNAME = "SeqNameFilter",
+                 SEQSTRAND = "SeqStrandFilter",
+                 TXNAME = "TxIdFilter",
+                 SYMBOL = "SymbolFilter")
+    if (withProteins) {
+        filters <- c(filters,
+                     PROTEINID = "ProteinIdFilter",
+                     UNIPROTID = "UniprotFilter",
+                     PROTEINDOMAINID = "ProtDomIdFilter")
+    }
     return(filters)
 }
-filterForKeytype <- function(keytype){
-    filters <- .keytype2FilterMapping()
+filterForKeytype <- function(keytype, x, vals){
+    if (missing(vals))
+        vals <- 1
+    if (!missing(x)) {
+        withProts <- hasProteinData(x)
+    } else {
+        withProts <- FALSE
+    }
+    filters <- .keytype2FilterMapping(withProts)
     if(any(names(filters) == keytype)){
-        filt <- new(filters[keytype])
+        filt <- do.call(filters[keytype], args = list(value = vals))
+        ## filt <- new(filters[keytype])
         return(filt)
     }else{
         stop("No filter for that keytype!")
@@ -104,28 +123,45 @@ filterForKeytype <- function(keytype){
 ##
 ####------------------------------------------------------------
 setMethod("keys", "EnsDb",
-          function(x, keytype, filter,...){
+          function(x, keytype, filter, ...){
               if(missing(keytype))
                   keytype <- "GENEID"
               if(missing(filter))
-                  filter <- list()
-              if(is(filter, "BasicFilter"))
-                  filter <- list(filter)
+                  filter <- AnnotationFilterList()
+              filter <- .processFilterParam(filter, x)
               keyt <- keytypes(x)
+              if (length(keytype) > 1) {
+                  keytype <- keytype[1]
+                  warning("Using only first provided keytype.")
+              }
+              if (!any(keyt == keytype))
+                  stop("keytype '", keytype, "' not supported! ",
+                       "Allowed choices are: ",
+                       paste0("'", keyt ,"'", collapse = ", "), ".")
               keytype <- match.arg(keytype, keyt)
               ## Map the keytype to the appropriate column name.
               dbColumn <- ensDbColumnForColumn(x, keytype)
               ## Perform the query.
-              res <- getWhat(x, columns=dbColumn, filter=filter)[, dbColumn]
+              res <- getWhat(x, columns = dbColumn, filter = filter)[, dbColumn]
               return(res)
           })
 
 
-####============================================================
-##  select method
+############################################################
+## select method
 ##
-##
-####------------------------------------------------------------
+##  We have to be carefull, if the database contains protein annotations too:
+##  o If the keys are DNA/RNA related, start from a DNA/RNA related table.
+##  o if keys are protein related: start from a protein column.
+##  Reason is that we do have only protein annotations for protein coding genes
+##  and no annotation for the remaining. Thus the type of the join (left join,
+##  left outer join) is crucial, as well as the table with which we start the
+##  query!
+##  What if we provide more than one filter?
+##  a) GenenameFilter and ProteinidFilter: doesn't really matter from which table
+##     we start, because the query will only return results with protein
+##     annotions. -> if there is one DNA/RNA related filter: don't do anything.
+##  b) Only protein filters: start from the highest protein table.
 setMethod("select", "EnsDb",
           function(x, keys, columns, keytype, ...) {
               if (missing(keys))
@@ -147,8 +183,9 @@ setMethod("select", "EnsDb",
     if (all(notAvailable))
         stop("None of the specified columns are avaliable in the database!")
     if (any(notAvailable)){
-        warning("The following columns are not available in the database and have",
-                " thus been removed: ", paste(columns[notAvailable], collapse = ", "))
+        warning("The following columns are not available in the database and",
+                " have thus been removed: ",
+                paste(columns[notAvailable], collapse = ", "))
         columns <- columns[!notAvailable]
     }
     ## keys:
@@ -156,79 +193,92 @@ setMethod("select", "EnsDb",
         ## Get everything from the database...
         keys <- list()
     } else {
-        if (!(is(keys, "character") | is(keys, "list") | is(keys, "BasicFilter")))
-            stop("Argument keys should be a character vector, an object extending BasicFilter ",
-                 "or a list of objects extending BasicFilter.")
-        if (is(keys, "list")) {
-            if (!all(vapply(keys, is, logical(1L), "BasicFilter")))
-                stop("If keys is a list it should be a list of objects extending BasicFilter!")
-        }
-        if (is(keys, "BasicFilter")) {
-            keys <- list(keys)
-        }
+        if (!(is(keys, "character") | is(keys, "list") | is(keys, "formula") |
+              is(keys, "AnnotationFilter") | is(keys, "AnnotationFilterList")))
+            stop("Argument keys should be a character vector, an object",
+                 " extending AnnotationFilter, a filter expression",
+                 " or an AnnotationFilterList.")
         if (is(keys, "character")) {
             if (is.null(keytype)) {
-                stop("Argument keytype is mandatory if keys is a character vector!")
+                stop("Argument keytype is mandatory if keys is a",
+                     " character vector!")
             }
             ## Check also keytype:
             if (!(keytype %in% keytypes(x)))
                 stop("keytype ", keytype, " not available in the database.",
                      " Use keytypes method to list all available keytypes.")
             ## Generate a filter object for the filters.
-            keyFilter <- filterForKeytype(keytype)
-            value(keyFilter) <- keys
+            keyFilter <- filterForKeytype(keytype, x, vals = keys)
+            ## value(keyFilter) <- keys
+            ## keyFilter@value <- keys
             keys <- list(keyFilter)
             ## Add also the keytype itself to the columns.
             if (!any(columns == keytype))
                 columns <- c(keytype, columns)
         }
+        ## Check and fix filter.
+        keys <- .processFilterParam(keys, x)
     }
-    ## Map the columns to column names we have in the database and add filter columns too.
+    ## Map the columns to column names we have in the database and
+    ## add filter columns too.
     ensCols <- unique(c(ensDbColumnForColumn(x, columns),
                         addFilterColumns(character(), filter = keys, x)))
+    ## TODO @jo: Do we have to check that we are allowed to have protein filters
+    ##           or columns?
     ## OK, now perform the query given the filters we've got.
-    res <- getWhat(x, columns = ensCols, filter = keys)
+    ## Check if keys does only contain protein annotation columns; in that case
+    ## select one of tables "protein", "uniprot", "protein_domain" in that order
+    ## if (all(unlist(lapply(keys, isProteinFilter)))) {
+    if (all(isProteinFilter(keys))) {
+        startWith <- "protein_domain"
+        if (any(unlist(lapply(keys, function(z) is(z, "UniprotFilter")))))
+            startWith <- "uniprot"
+        if (any(unlist(lapply(keys, function(z) is(z, "ProteinIdFilter")))))
+            startWith <- "protein"
+    } else {
+        startWith <- NULL
+    }
+    ## Otherwise set startWith to NULL
+    res <- getWhat(x, columns = ensCols, filter = keys, startWith = startWith)
     ## Order results if length of filters is 1.
     if (length(keys) == 1) {
         ## Define the filters on which we could sort.
-        sortFilts <- c("GenenameFilter", "GeneidFilter", "EntrezidFilter", "GenebiotypeFilter",
-                       "SymbolFilter", "TxidFilter", "TxbiotypeFilter", "ExonidFilter",
-                       "ExonrankFilter", "SeqnameFilter")
+        sortFilts <- c("GenenameFilter", "GeneIdFilter", "EntrezFilter",
+                       "GeneBiotypeFilter", "SymbolFilter", "TxIdFilter",
+                       "TxBiotypeFilter", "ExonIdFilter", "ExonRankFilter",
+                       "SeqNameFilter")
         if (class(keys[[1]]) %in% sortFilts) {
             keyvals <- value(keys[[1]])
             ## Handle symlink Filter differently:
             if (is(keys[[1]], "SymbolFilter")) {
-                sortCol <- column(keys[[1]])
+                ## sortCol <- ensDbColumn(keys[[1]])
+                sortCol <- keys[[1]]@field
             } else {
-                sortCol <- removePrefix(column(keys[[1]], x))
+                sortCol <- ensDbColumn(keys[[1]])
+                ## sortCol <- removePrefix(ensDbColumn(keys[[1]], x))
             }
             res <- res[order(match(res[, sortCol], keyvals)), ]
         }
     } else {
         ## Show a mild warning message
-        message("Note: ordering of the results might not match ordering of keys!")
+        message(paste0("Note: ordering of the results might not match ordering",
+                       " of keys!"))
     }
     colMap <- .getColMappings(x)
     colnames(res) <- colMap[colnames(res)]
     rownames(res) <- NULL
     if (returnFilterColumns(x))
         return(res)
-    ## ## Now, if we've got a "TXNAME" in columns, we have to replace at least one of the "TXID"s
-    ## ## in the colnames...
-    ## if(any(columns == "TXNAME"))
-    ##     colnames(res)[match("TXID", colnames(res))] <- "TXNAME"
-    return(res[, columns])
+    res[, columns]
 }
 
-
-####============================================================
+############################################################
 ##  mapIds method
 ##
 ##  maps the submitted keys (names of the returned vector) to values
 ##  of the column specified by column.
 ##  x, key, column, keytype, ..., multiVals
-####------------------------------------------------------------
-setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals){
+setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals) {
     if(missing(keys))
         keys <- NULL
     if(missing(column))
@@ -237,38 +287,53 @@ setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals){
         keytype <- NULL
     if(missing(multiVals))
         multiVals <- NULL
-    return(.mapIds(x=x, keys=keys, column=column, keytype=keytype, multiVals=multiVals, ...))
+    return(.mapIds(x = x, keys = keys, column = column, keytype = keytype,
+                   multiVals = multiVals, ...))
 })
 ## Other methods: saveDb, species, dbfile, dbconn, taxonomyId
-.mapIds <- function(x, keys=NULL, column=NULL, keytype=NULL, ..., multiVals=NULL){
-    if(is.null(keys))
+.mapIds <- function(x, keys = NULL, column = NULL, keytype = NULL, ...,
+                    multiVals = NULL) {
+    if (is.null(keys))
         stop("Argument keys has to be provided!")
-    if(!(is(keys, "character") | is(keys, "list") | is(keys, "BasicFilter")))
-        stop("Argument keys should be a character vector, an object extending BasicFilter ",
-             "or a list of objects extending BasicFilter.")
-    if(is.null(column))
+    ## if (!(is(keys, "character") | is(keys, "list") |
+    ##       is(keys, "AnnotationFilter")))
+    ##     stop("Argument keys should be a character vector, an object extending",
+    ##          " AnnotationFilter or a list of objects extending AnnotationFilter.")
+    if (is.null(column))
         column <- "GENEID"
     ## Have to specify the columns argument. Has to be keytype and column.
-    if(is(keys, "character")){
-        if(is.null(keytype))
+    if (is(keys, "character")){
+        if (is.null(keytype))
             stop("Argument keytype is mandatory if keys is a character vector!")
         columns <- c(keytype, column)
-    }
-    if(is(keys, "list") | is(keys, "BasicFilter")){
-        if(is(keys, "list")){
-            if(length(keys) > 1)
-                warning("Got ", length(keys), " filter objects.",
-                        " Will use the keys of the first for the mapping!")
-            cn <- class(keys[[1]])[1]
-        }else{
-            cn <- class(keys)[1]
-        }
+    } else {
+        ## Test if we can convert the filter. Returns ALWAYS an
+        ## AnnotationFilterList
+        keys <- .processFilterParam(keys, x)
+        if(length(keys) > 1)
+            warning("Got ", length(keys), " filter objects.",
+                    " Will use the keys of the first for the mapping!")
+        cn <- class(keys[[1]])[1]
         ## Use the first element to determine the keytype...
         mapping <- .keytype2FilterMapping()
         columns <- c(names(mapping)[mapping == cn], column)
         keytype <- NULL
     }
-    res <- select(x, keys=keys, columns=columns, keytype=keytype)
+    ## if(is(keys, "list") | is(keys, "AnnotationFilter")){
+    ##     if(is(keys, "list")){
+    ##         if(length(keys) > 1)
+    ##             warning("Got ", length(keys), " filter objects.",
+    ##                     " Will use the keys of the first for the mapping!")
+    ##         cn <- class(keys[[1]])[1]
+    ##     }else{
+    ##         cn <- class(keys)[1]
+    ##     }
+    ##     ## Use the first element to determine the keytype...
+    ##     mapping <- .keytype2FilterMapping()
+    ##     columns <- c(names(mapping)[mapping == cn], column)
+    ##     keytype <- NULL
+    ## }
+    res <- select(x, keys = keys, columns = columns, keytype = keytype)
     if(nrow(res) == 0)
         return(character())
     ## Handling multiVals.
@@ -276,13 +341,6 @@ setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals){
         multiVals <- "first"
     if(is(multiVals, "function"))
         stop("Not yet implemented!")
-    ## Eventually re-order the data.frame in the same order than the keys...
-    ## That's amazingly slow!!!
-    ## if(is.character(keys)){
-    ##     res <- split(res, f=factor(res[, 1], levels=keys))
-    ##     res <- do.call(rbind, res)
-    ##     rownames(res) <- NULL
-    ## }
     if(is.character(keys)){
         theNames <- keys
     }else{
